#include"jltest.h"

#include<qpushbutton.h>
#include<qlayout.h>
#include<qcheckbox.h>
#include<qmessagebox.h>
#include<qdom.h>
#include"kmp/stream.h"
#include"kmp/dtcp.h"
#include"psiaccount.h"
#include"busywidget.h"


class JLTestDlg::Private
{
public:
	Private() {}

	PsiAccount *pa;
	Jid jid;
	QCheckBox *ck_passive;
	BusyWidget *busy;
	QPushButton *pb_test;
	DTCPConnection *c;
	QLabel *l;
	int mode;
};

JLTestDlg::JLTestDlg(const Jid &j, DTCPConnection *c, PsiAccount *pa)
:QDialog(0, 0, false, WDestructiveClose)
{
	d = new Private;
	d->pa = pa;
	d->jid = j;

	if(c) {
		d->c = c;
		d->mode = 1;
	}
	else {
		d->c = 0;
		d->mode = 0;
	}

	if(d->mode == 0)
		setCaption("DTCP Connect");
	else
		setCaption("DTCP Accept");

	QVBoxLayout *vb = new QVBoxLayout(this, 8);
	d->l = new QLabel(d->jid.full(), this);
	vb->addWidget(d->l);
	if(d->mode == 1) {
		QLabel *l = new QLabel(QString("Accept DTCP Connection?\nComment:\n------\n%2\n------").arg(Stream::elemToString(d->c->comment())), this);
		vb->addWidget(l);
	}
	d->ck_passive = new QCheckBox(this);
	vb->addWidget(d->ck_passive);
	d->ck_passive->setText(tr("Passive mode"));
	if(d->mode == 1) {
		d->ck_passive->setChecked(d->c->isPassive());
		d->ck_passive->setEnabled(false);
	}

	vb->addStretch(1);
	QHBoxLayout *hb = new QHBoxLayout(vb);
	d->busy = new BusyWidget(this);
	hb->addWidget(d->busy);
	d->pb_test = new QPushButton(this);
	if(d->mode == 0)
		d->pb_test->setText(tr("Connect"));
	else
		d->pb_test->setText(tr("Accept"));
	connect(d->pb_test, SIGNAL(clicked()), SLOT(start()));
	hb->addWidget(d->pb_test);
	QPushButton *pb_close = new QPushButton(tr("Close"), this);
	connect(pb_close, SIGNAL(clicked()), SLOT(close()));
	hb->addWidget(pb_close);
}

JLTestDlg::~JLTestDlg()
{
	delete d->c;
	delete d;
}

void JLTestDlg::start()
{
	if(d->mode == 0) {
		d->c = new DTCPConnection(d->pa->client()->dtcpManager());
		connect(d->c, SIGNAL(connected()), SLOT(dtcp_connected()));
		connect(d->c, SIGNAL(connectionClosed()), SLOT(dtcp_connectionClosed()));
		connect(d->c, SIGNAL(error(int)), SLOT(dtcp_error(int)));

		if(!d->c->connectToJid(d->jid, d->ck_passive->isChecked(), QDomElement())) {
			delete d->c;
			d->c = 0;
			QMessageBox::information(this, tr("error"), tr("Unable to perform DTCP Connect.  Perhaps you are trying passive mode when you were unable to bind to a port?"));
			return;
		}
	}
	else {
		connect(d->c, SIGNAL(connected()), SLOT(dtcp_connected()));
		connect(d->c, SIGNAL(connectionClosed()), SLOT(dtcp_connectionClosed()));
		connect(d->c, SIGNAL(error(int)), SLOT(dtcp_error(int)));

		d->c->accept();
	}

	d->busy->start();
	d->pb_test->setEnabled(false);
	d->ck_passive->setEnabled(false);
}

void JLTestDlg::dtcp_connected()
{
	d->l->setText(tr("%1 - Connected").arg(d->jid.full()));
	d->busy->stop();
}

void JLTestDlg::dtcp_connectionClosed()
{
	if(d->mode == 0) {
		d->pb_test->setEnabled(true);
		d->ck_passive->setEnabled(true);
	}

	QMessageBox::information(this, tr("error"), tr("Connection closed"));
	delete d->c;
	d->c = 0;

	if(d->mode == 1)
		close();
}

void JLTestDlg::dtcp_error(int x)
{
	d->busy->stop();
	if(d->mode == 0) {
		d->pb_test->setEnabled(true);
		d->ck_passive->setEnabled(true);
	}

	QString str;
	if(x == DTCPConnection::ErrAsk)
		str = tr("Error during request");
	else if(x == DTCPConnection::ErrEstablish)
		str = tr("Error during establish");
	else
		str = tr("Socket error");

	QMessageBox::information(this, tr("error"), str);
	delete d->c;
	d->c = 0;

	if(d->mode == 1)
		close();
}
