/****************************************************************************
** mainwin.cpp - the main window.  holds contactlist and buttons.
** Copyright (C) 2001, 2002  Justin Karneges
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
**
****************************************************************************/

#include"mainwin.h"

#include<qmessagebox.h>
#include<qiconset.h>
#include<qtooltip.h>
#include<qstyle.h>
#include<qapplication.h>
#include<qptrlist.h>
#include<qtimer.h>
#include<qobjectlist.h>
#include"kmp/types.h"
#include"common.h"
#include"showtextdlg.h"
#include"psicon.h"
#include"psiaccount.h"

using namespace Jabber;

// deletes submenus in a popupmenu
void qpopupmenuclear(QPopupMenu *p)
{
	while(p->count()) {
		QMenuItem *item = p->findItem(p->idAt(0));
		QPopupMenu *popup = item->popup();
		p->removeItemAt(0);

		if(popup)
			delete popup;
	}
}


//*******************************************************
//  MainWin
//*******************************************************
MainWin::MainWin(bool _onTop, bool _asTool, PsiCon *_psi, const char *name)
:QWidget(0, name, 0 | (_onTop ? WStyle_StaysOnTop: 0) | (_asTool ? WStyle_Tool: 0))
{
	setIcon(is.status(STATUS_OFFLINE));

	psi = _psi;
	onTop = _onTop;
	asTool = _asTool;

	// sbState:
	//   -1 : connect
	// >= 0 : STATUS_*  
	sbState = STATUS_OFFLINE;
	lastStatus = -2;

	nextAmount = 0;
	nextAnim = 0;
	tray = 0;
	trayMenu = 0;
	statusTip = "";
	infoString = "";
	nickname = "";

	vb_main = new QVBoxLayout(this, 2);
	QHBoxLayout *hb_tools = new QHBoxLayout(vb_main);

	tb_offline = new MToolButton(this);
	tb_offline->setToggleButton(TRUE);
	tb_offline->setOn(TRUE);
	hb_tools->addWidget(tb_offline);

	tb_away = new MToolButton(this);
	tb_away->setToggleButton(TRUE);
	tb_away->setOn(TRUE);
	hb_tools->addWidget(tb_away);

	tb_agents = new MToolButton(this);
	tb_agents->setToggleButton(TRUE);
	tb_agents->setOn(TRUE);
	hb_tools->addWidget(tb_agents);

	setToolBarIcons();

	hb_tools->addStretch(1);

	QToolTip::add(tb_offline, tr("Show Offline Contacts"));
	QToolTip::add(tb_away, tr("Show Away/XA/DnD"));
	QToolTip::add(tb_agents, tr("Show Agents/Transports"));

	cvlist = new ContactView(this);
	connect(tb_offline, SIGNAL(toggled(bool)), cvlist, SLOT(setShowOffline(bool)));
	connect(tb_away, SIGNAL(toggled(bool)), cvlist, SLOT(setShowAway(bool)));
	connect(tb_agents, SIGNAL(toggled(bool)), cvlist, SLOT(setShowAgents(bool)));
	connect(cvlist, SIGNAL(showOffline(bool)), tb_offline, SLOT(setPressed(bool)));
	connect(cvlist, SIGNAL(showAway(bool)), tb_away, SLOT(setPressed(bool)));
	connect(cvlist, SIGNAL(showAgents(bool)), tb_agents, SLOT(setPressed(bool)));
	vb_main->addWidget(cvlist, 1);

	QHBoxLayout *hb1 = new QHBoxLayout(vb_main);
	lb_info = new MLabel(this);
	connect(lb_info, SIGNAL(clicked(int)), SLOT(statusClicked(int)));
	connect(lb_info, SIGNAL(doubleClicked()), SLOT(doRecvNextEvent()));
	lb_info->setMinimumWidth(48);
	lb_info->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed ) );
	lb_info->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	hb1->addWidget(lb_info);
	lb_ssl = new QLabel(this);
	lb_ssl->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	setUsingSSL(FALSE);
	hb1->addWidget(lb_ssl);
	lb_info->hide();
	lb_ssl->hide();

	hb_status = new QHBoxLayout(vb_main);
	pb_options = new QPushButton(this);
	pb_options->setPixmap(is.psimain);
	pb_options->setSizePolicy( QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Expanding ) );
	hb_status->addWidget(pb_options);

	pb_status = new MPushButton(this);
	pb_status->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed ) );
	decorateButton(STATUS_OFFLINE);
	hb_status->addWidget(pb_status);

	add_act = new MAction(QIconSet(is.add), tr("&Add a contact"), 0, psi);
	connect(add_act, SIGNAL(activated(PsiAccount *, int)), SLOT(activatedAccOption(PsiAccount *, int)));

	man_act = new MAction(QIconSet(transport2icon("", STATUS_ONLINE)), tr("&Manage services"), 1, psi);
	connect(man_act, SIGNAL(activated(PsiAccount *, int)), SLOT(activatedAccOption(PsiAccount *, int)));

	lw_act = new MAction(tr("&XML Console"), 2, psi);
	connect(lw_act, SIGNAL(activated(PsiAccount *, int)), SLOT(activatedAccOption(PsiAccount *, int)));

	statusMenu = new QPopupMenu(this);
	optionsMenu = new QPopupMenu(this);
	buildStatusMenu();
	buildOptionsMenu();
	connect(statusMenu, SIGNAL(aboutToShow()), SLOT(buildStatusMenu()));
	connect(statusMenu, SIGNAL(activated(int)), SLOT(activatedStatusMenu(int)));
	connect(optionsMenu, SIGNAL(aboutToShow()), SLOT(buildOptionsMenu()));
	connect(optionsMenu, SIGNAL(activated(int)), SLOT(activatedOptionsMenu(int)));

	pb_options->setPopup(optionsMenu);
	pb_status->setPopup(statusMenu);

#ifdef Q_WS_MAC
	vb_main->addSpacing(16);
#endif

	updateCaption();

	connect(psi, SIGNAL(accountCountChanged()), SLOT(numAccountsChanged()));
	numAccountsChanged();
}

MainWin::~MainWin()
{
	if(tray) {
		delete tray;
		tray = 0;
		delete trayMenu;
		trayMenu = 0;
	}
}

void MainWin::setWindowOpts(bool _onTop, bool _asTool)
{
	if(_onTop == onTop && _asTool == asTool)
		return;

	onTop = _onTop;
	asTool = _asTool;

	WFlags flags = 0;
	if(onTop)
		flags |= WStyle_StaysOnTop;
	if(asTool)
		flags |= WStyle_Tool;

	QPoint p = pos();
	reparent(parentWidget(), flags, p, FALSE);
	move(p);
	show();
}

void MainWin::setUseDock(bool use)
{
	if(use == false) {
		if(tray) {
			delete tray;
			tray = 0;
			delete trayMenu;
			trayMenu = 0;
		}
		return;
	}

	if(tray)
		return;

	trayMenu = new QPopupMenu;
	connect(trayMenu, SIGNAL(aboutToShow()), SLOT(buildTrayMenu()));

	tray = new MTray(is.status(STATUS_OFFLINE), "Psi", trayMenu);
	connect(tray, SIGNAL(clicked(const QPoint &, int)), SLOT(trayClicked(const QPoint &, int)));
	connect(tray, SIGNAL(doubleClicked(const QPoint &)), SLOT(trayDoubleClicked()));
	connect(tray, SIGNAL(closed()), SIGNAL(closeProgram()));

	updateReadNext(nextAnim, nextAmount);

	tray->show();
}

void MainWin::setInfo(const QString &str)
{
	infoString = str;

	if(nextAmount == 0)
		lb_info->setText(QString("<nobr>") + infoString + "</nobr>");
}

void MainWin::setUsingSSL(bool use)
{
	usingSSL = use;
	lb_ssl->setPixmap(usingSSL ? is.ssl_yes : is.ssl_no);
	QToolTip::add(lb_ssl, usingSSL ? tr("Connection is encrypted") : tr("Connection is not encrypted"));
}

void MainWin::buildStatusMenu()
{
	//statusMenu->clear();
	qpopupmenuclear(statusMenu);

	statusMenu->insertItem(QIconSet(is.status(STATUS_ONLINE)),  status2txt(STATUS_ONLINE),      STATUS_ONLINE);
	statusMenu->insertItem(QIconSet(is.status(STATUS_AWAY)),    status2txt(STATUS_AWAY),        STATUS_AWAY);
	statusMenu->insertItem(QIconSet(is.status(STATUS_XA)),      status2txt(STATUS_XA),          STATUS_XA);
	statusMenu->insertItem(QIconSet(is.status(STATUS_DND)),     status2txt(STATUS_DND),         STATUS_DND);
	statusMenu->insertSeparator();
	statusMenu->insertItem(QIconSet(is.status(STATUS_OFFLINE)), status2txt(STATUS_OFFLINE),     STATUS_OFFLINE);
}

void MainWin::activatedStatusMenu(int id)
{
	statusChanged(id);
}

void MainWin::buildOptionsMenu()
{
	add_act->removeFrom(optionsMenu);
	man_act->removeFrom(optionsMenu);
	lw_act->removeFrom(optionsMenu);

	// clear the menu and submenus
	qpopupmenuclear(optionsMenu);

	// help menu
	QPopupMenu *helpMenu = new QPopupMenu(optionsMenu);
	helpMenu->insertItem(tr("&ReadMe"), 13);
	//helpMenu->insertItem(tr("&Hints"), 14);
	helpMenu->insertItem(tr("&License"), 11);
	helpMenu->insertItem(tr("&Warning"), 12);
	helpMenu->insertSeparator();
	helpMenu->insertItem(tr("&About"), 7);
	helpMenu->insertItem(tr("About &Qt"), 8);
	//helpMenu->setItemEnabled(14, FALSE);
	connect(helpMenu, SIGNAL(activated(int)), SLOT(activatedOptionsMenu(int)));

	// options menu
	add_act->addTo(optionsMenu);
	optionsMenu->insertItem(QIconSet(is.send),  tr("New &blank message"), 1);
	man_act->addTo(optionsMenu);
	optionsMenu->insertItem(QIconSet(is.groupchat), tr("Join &Groupchat"), 17);
	//optionsMenu->insertItem(tr("File sharing"), 4);
	//optionsMenu->setItemEnabled(4, FALSE);
	optionsMenu->insertItem(QIconSet(is.account), tr("Acc&ount Setup"), 16);
	optionsMenu->insertItem(tr("&Options"),             9);

	lw_act->addTo(optionsMenu);

	optionsMenu->insertItem(QIconSet(is.profile), tr("&Change profile"),3);
	optionsMenu->insertItem(tr("Play &sounds"), 2);
	optionsMenu->setItemChecked(2, useSound);
	optionsMenu->insertSeparator();
	optionsMenu->insertItem(tr("&Help"), helpMenu);
	optionsMenu->insertItem(tr("&Quit"),                 5);

	if(psi->accountList().isEmpty()) {
		optionsMenu->setItemEnabled(0, false);
		optionsMenu->setItemEnabled(1, false);
		optionsMenu->setItemEnabled(15, false);
		optionsMenu->setItemEnabled(17, false);
	}
}

void MainWin::testme()
{
	optionsMenu->setItemEnabled(10, false);
}

void MainWin::activatedOptionsMenu(int id)
{
	QString dstr; dstr.sprintf("mainwin: id=%d\n", id);
	//pdb(DEBUG_MAINWIN, dstr);

	if(id == 1)
		blankMessage();
	else if(id == 2) {
		if(useSound)
			useSound = FALSE;
		else
			useSound = TRUE;
	}
	else if(id == 3)
		changeProfile();
	else if(id == 4)
		doFileSharing();
	else if(id == 5) {
		tryCloseProgram();
	}
	else if(id == 6)
		accountInfo();
	else if(id == 7) {
		QString content = QString("<h3>%1 v%2</h3>\n").arg(PROG_NAME).arg(PROG_VERSION);
		content += "<br>\n";
		content += "Copyright (C) 2001,2002<br>\n";
		content += "By Justin Karneges<br>\n";
		content += "infiniti@affinix.com<br>\n";
		content += "http://psi.affinix.com/<br>\n";
		content += "<br>\n";
		content += "Psi is free software, covered by the GNU General Public License, and you are welcome to change it and/or distribute copies of it under certain conditions.  See Help->License for details.<br>\n";
		content += "<br>\n";
		content += "A capable Jabber client written in C++ using the Qt GUI Toolkit.<br>\n";
		content += "<br>\n";
		content += "Mac port contributed by Bill Myers.<br>\n";
		content += "Graphics by Jason Kim, Justin Karneges, Hideaki Omuro, and other contributers.<br>\n";

		AboutDlg *w = new AboutDlg(*pix_icon_48, content, this);
		w->setCaption(tr("About"));
		w->exec();
		delete w;
	}
	else if(id == 8) {
		QMessageBox::aboutQt(this);
	}
	else if(id == 9) {
		doOptions();
	}
	else if(id == 11) {
		ShowTextDlg *w = new ShowTextDlg(g.pathBase + "/COPYING");
		w->setCaption(CAP(tr("License")));
		w->show();
	}
	else if(id == 12) {
		unfinishedSoftware();
	}
	else if(id == 13) {
		// readme
#ifdef Q_WS_WIN
		ShowTextDlg *w = new ShowTextDlg(g.pathBase + "/readme.txt");
#else
		ShowTextDlg *w = new ShowTextDlg(g.pathBase + "/README");
#endif
		w->setCaption(CAP(tr("ReadMe")));
		w->show();
	}
	else if(id == 14) {
		// hints
	}
	else if(id == 16) {
		doManageAccounts();
	}
	else if(id == 17) {
		doGroupChat();
	}
}

void MainWin::activatedAccOption(PsiAccount *pa, int x)
{
	if(x == 0)
		pa->openAddUserDlg();
	else if(x == 1)
		pa->doManageServices();
	else if(x == 2)
		pa->showXmlConsole();
}

void MainWin::buildTrayMenu()
{
	if(!trayMenu)
		return;
	trayMenu->clear();

	if(nextAmount > 0) {
		trayMenu->insertItem(tr("Receive next event"), this, SLOT(doRecvNextEvent()));
		trayMenu->insertSeparator();
	}

	if(isHidden())
		trayMenu->insertItem(tr("Un&hide"), this, SLOT(trayShow()));
	else
		trayMenu->insertItem(tr("&Hide"), this, SLOT(trayHide()));
	trayMenu->insertItem(tr("&Psi"), optionsMenu);
	trayMenu->insertItem(tr("&Status"), statusMenu);
	trayMenu->insertSeparator();
	trayMenu->insertItem(tr("&Quit"), this, SLOT(tryCloseProgram()));
}

void MainWin::decorateButton(int status)
{
	if(lastStatus == status)
		return;
	lastStatus = status;

	QIconSet icon;
	icon.setPixmap(is.status(status), QIconSet::Small);

	if(status == -1) {
		pb_status->setAnim(&is.conn);
		pb_status->setText(tr("Connecting"));

		setIcon(is.status(STATUS_OFFLINE));
	}
	else {
		pb_status->setAnim(0);
		pb_status->setIconSet(icon);
		pb_status->setText(status2txt(status));

		setIcon(is.status(status));
	}

	updateTray();
}

bool MainWin::askQuit()
{
	return TRUE;
}

void MainWin::tryCloseProgram()
{
	if(askQuit())
		closeProgram();
}

void MainWin::closeEvent(QCloseEvent *e)
{
	if(tray) {
		hide();
		return;
	}

	if(!askQuit())
		return;

        closeProgram();
	e->accept();
}

void MainWin::keyPressEvent(QKeyEvent *e)
{
	if(tray && e->key() == Key_Escape) {
		close();
		e->accept();
		return;
	}

	QWidget::keyPressEvent(e);
}

void MainWin::updateCaption()
{
	QString str = "";

	if(nextAmount > 0)
		str += "* ";

	if(nickname.isEmpty())
		str += PROG_NAME;
	else
		str += nickname;

	if(str == caption())
		return;

	setCaption(str);
}

void MainWin::setToolBarIcons()
{
	tb_offline->setIconSet(QIconSet(is.status(STATUS_OFFLINE)));
	tb_away->setIconSet(QIconSet(is.status(STATUS_AWAY)));
	tb_agents->setIconSet(QIconSet(transport2icon("", STATUS_ONLINE)));
}

void MainWin::optionsUpdate()
{
	int status = lastStatus;
	lastStatus = -2;
	decorateButton(status);
	setToolBarIcons();
	pb_options->setPixmap(is.psimain);

	updateTray();
}

void MainWin::trayClicked(const QPoint &, int button)
{
	if(option.dockDCstyle)
		return;

	if(button == MidButton) {
		doRecvNextEvent();
		return;
	}

	if(isHidden())
		trayShow();
	else
		trayHide();
}

void MainWin::trayDoubleClicked()
{
	if(!option.dockDCstyle)
		return;

	if(nextAmount > 0) {
		doRecvNextEvent();
		return;
	}

	if(isHidden())
		trayShow();
	else
		trayHide();
}

void MainWin::trayShow()
{
	show();
	bringToFront(this);
}

void MainWin::trayHide()
{
	hide();
}

void MainWin::updateReadNext(Anim *anim, int amount)
{
#ifdef Q_WS_MAC
	if(amount > nextAmount)
		stopDockTileBounce();
	else
		bounceDockTile();
#endif

	nextAnim = anim;
	if(anim == 0)
		nextAmount = 0;
	else
		nextAmount = amount;

	if(nextAmount <= 0) {
		lb_info->hide();
		lb_info->setText("");
	}
	else {
		lb_info->setText(QString("<nobr><b>") + numEventsString(nextAmount) + "</b></nobr>");
		lb_info->show();
	}

	updateTray();
	updateCaption();
}

QString MainWin::numEventsString(int x) const
{
	QString s;
	if(x <= 0)
		s = "";
	else if(x == 1)
		s = tr("1 event received");
	else
		s = tr("%1 events received").arg(x);

	return s;
}

void MainWin::updateTray()
{
	if(!tray)
		return;

	tray->setToolTip(numEventsString(nextAmount));

	if(nextAmount > 0)
		tray->setAnim(nextAnim);
	else if(lastStatus == -1)
		tray->setAnim(&is.conn);
	else
		tray->setImage(is.status(lastStatus));
}

void MainWin::doRecvNextEvent()
{
	recvNextEvent();
}

void MainWin::statusClicked(int x)
{
	if(x == MidButton)
		recvNextEvent();
}

void MainWin::numAccountsChanged()
{
	pb_status->setEnabled(psi->accountList().count() > 0);
}


#if defined(Q_WS_WIN)
#include<windows.h>
void MainWin::showNoFocus()
{
	clearWState( WState_ForceHide );

	if ( testWState(WState_Visible) ) {
		SetWindowPos(winId(),HWND_TOPMOST,0,0,0,0, SWP_NOACTIVATE|SWP_NOMOVE|SWP_NOSIZE);
		if(!onTop)
			SetWindowPos(winId(),HWND_NOTOPMOST,0,0,0,0, SWP_NOACTIVATE|SWP_NOMOVE|SWP_NOSIZE);
		return; // nothing to do
	}

	if ( isTopLevel() && !testWState( WState_Resized ) )  {
		// do this before sending the posted resize events. Otherwise
		// the layout would catch the resize event and may expand the
		// minimum size.
		QSize s = sizeHint();
		QSizePolicy::ExpandData exp;
#ifndef QT_NO_LAYOUT
		if ( layout() ) {
			if ( layout()->hasHeightForWidth() )
				s.setHeight( layout()->totalHeightForWidth( s.width() ) );
			exp =  layout()->expanding();
		} else
#endif
		{
			if ( sizePolicy().hasHeightForWidth() )
				s.setHeight( heightForWidth( s.width() ) );
			exp = sizePolicy().expanding();
		}
		if ( exp & QSizePolicy::Horizontally )
			s.setWidth( QMAX( s.width(), 200 ) );
		if ( exp & QSizePolicy::Vertically )
			s.setHeight( QMAX( s.height(), 150 ) );
		QRect screen = QApplication::desktop()->screenGeometry( QApplication::desktop()->screenNumber( pos() ) );
		s.setWidth( QMIN( s.width(), screen.width()*2/3 ) );
		s.setHeight( QMIN( s.height(), screen.height()*2/3 ) );
		if ( !s.isEmpty() )
			resize( s );
	}

	QApplication::sendPostedEvents( this, QEvent::Move );
	QApplication::sendPostedEvents( this, QEvent::Resize );

	setWState( WState_Visible );

	if ( testWFlags(WStyle_Tool) || isPopup() ) {
		raise();
	} else if ( testWFlags(WType_TopLevel) ) {
		while ( QApplication::activePopupWidget() )
		QApplication::activePopupWidget()->close();
	}

	if ( !testWState(WState_Polished) )
		polish();

	if ( children() ) {
		QObjectListIt it(*children());
		register QObject *object;
		QWidget *widget;
		while ( it ) {				// show all widget children
			object = it.current();		//   (except popups and other toplevels)
			++it;
			if ( object->isWidgetType() ) {
				widget = (QWidget*)object;
				if ( !widget->isHidden() && !widget->isTopLevel() )
				widget->show();
			}
		}
	}

#if defined(QT_ACCESSIBILITY_SUPPORT)
	QAccessible::updateAccessibility( this, 0, QAccessible::ObjectShow );
#endif

	SetWindowPos(winId(),HWND_TOP,0,0,0,0, SWP_NOACTIVATE|SWP_NOMOVE|SWP_NOSIZE|SWP_SHOWWINDOW);
	UpdateWindow(winId());
}

#else

void MainWin::showNoFocus()
{
	showNormal();
	raise();
}

#endif


//*******************************************************
//  MPushButton
//*******************************************************
MPushButton::MPushButton(QWidget *parent, const char *name)
:AnimatedButton(parent, name, true)
{
	setMinimumWidth(48);
	setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed));

	hasTip = FALSE;
}

void MPushButton::setText(const QString &_text)
{
	if(hasTip) {
		QToolTip::remove(this);
		hasTip = FALSE;
	}

	QPushButton::setText(_text);
}

void MPushButton::drawButtonLabel(QPainter *p)
{
	// crazy code ahead!  watch out for potholes and deer.

	// this gets us the width of the "text area" on the button.
	// adapted from qt/src/styles/qcommonstyle.cpp and qt/src/widgets/qpushbutton.cpp
	QRect r = style().subRect(QStyle::SR_PushButtonContents, this);
	if(isMenuButton())
		r.setWidth(r.width() - style().pixelMetric(QStyle::PM_MenuButtonIndicator, this));
	if(iconSet() && !iconSet()->isNull())
		r.setWidth(r.width() - (iconSet()->pixmap(QIconSet::Small, QIconSet::Normal, QIconSet::Off).width() + 4));

	// font metrics
	QFontMetrics fm(font());

	// w1 = width of button text, w2 = width of text area
	int w1 = fm.width(text());
	int w2 = r.width();

	// backup original text
	QString oldtext = text();

	// button text larger than what will fit?
	if(w1 > w2) {
		if(!hasTip) {
			QToolTip::add(this, text());
			hasTip = TRUE;
		}

		// make a string that fits
		bool found = FALSE;
		QString newtext;
		int n;
		for(n = oldtext.length(); n > 0; --n) {
			if(fm.width(oldtext, n) < w2) {
				found = TRUE;
				break;
			}
		}
		if(found)
			newtext = oldtext.mid(0, n);
		else
			newtext = "";

		// set the new text that fits.  updates must be off, or we recurse.
		setUpdatesEnabled(FALSE);
		QButton::setText(newtext);
		setUpdatesEnabled(TRUE);
	}
	else {
		if(hasTip) {
			QToolTip::remove(this);
			hasTip = FALSE;
		}
	}

	// draw!
	QPushButton::drawButtonLabel(p);

	// restore original button text now that we are done drawing.
	setUpdatesEnabled(FALSE);
	QButton::setText(oldtext);
	setUpdatesEnabled(TRUE);
}


//*******************************************************
//  MToolButton
//*******************************************************
MToolButton::MToolButton(QWidget *parent, const char *name)
:QToolButton(parent, name)
{
}

void MToolButton::setPressed(bool x)
{
	setOn(x);
}


//*******************************************************
//  MLabel
//*******************************************************
MLabel::MLabel(QWidget *parent, const char *name)
:QLabel(parent, name)
{
}

void MLabel::mouseReleaseEvent(QMouseEvent *e)
{
	clicked(e->button());
	e->ignore();
}

void MLabel::mouseDoubleClickEvent(QMouseEvent *e)
{
	if(e->button() == LeftButton)
		doubleClicked();

	e->ignore();
}


//*******************************************************
//  AboutDlg
//*******************************************************
AboutDlg::AboutDlg(const QPixmap &icon, const QString &content, QWidget *par)
:QDialog(par, 0, TRUE)
{
	QLabel *l;
	QVBoxLayout *vb = new QVBoxLayout(this, 8);
	QHBoxLayout *hb = new QHBoxLayout(vb);
	QVBoxLayout *col;

	col = new QVBoxLayout(hb);
	l = new QLabel(this);
	l->setPixmap(icon);
	col->addWidget(l);
	col->addStretch(1);

	col = new QVBoxLayout(hb);
	l = new QLabel(this);
	l->setText(content);
	col->addWidget(l);
	col->addStretch(1);

	hb = new QHBoxLayout(vb);
	hb->addStretch(1);
	QPushButton *pb_ok = new QPushButton(tr("&OK"), this);
	connect(pb_ok, SIGNAL(clicked()), SLOT(close()));
	hb->addWidget(pb_ok);
	hb->addStretch(1);
}


//----------------------------------------------------------------------------
// MAction
//----------------------------------------------------------------------------
class MAction::MenuItem
{
public:
	MenuItem() {}

	QPopupMenu *popup;
	int id;
};

class MAction::Private
{
public:
	Private() {}

	QIconSet iconSet;
	QString str;
	PsiCon *psi;
	int id;

	QPtrList<MenuItem> menuitems;
};

MAction::MAction(const QIconSet &i, const QString &s, int id, PsiCon *psi, QObject *parent)
:QObject(parent)
{
	init(i, s, id, psi);
}

MAction::MAction(const QString &s, int id, PsiCon *psi, QObject *parent)
:QObject(parent)
{
	init(QIconSet(), s, id, psi);
}

void MAction::init(const QIconSet &i, const QString &s, int id, PsiCon *psi)
{
	d = new Private;

	d->iconSet = i;
	d->str = s;
	d->psi = psi;
	d->id = id;
	d->menuitems.setAutoDelete(true);

	connect(psi, SIGNAL(accountCountChanged()), SLOT(numAccountsChanged()));
}

MAction::~MAction()
{
	// delete all menu items
	QPtrListIterator<MenuItem> it(d->menuitems);
	for(MenuItem *mi; (mi = it.current());) {
		QPopupMenu *pm = mi->popup;
		QMenuItem *i = pm->findItem(mi->id);
		if(i) {
			QPopupMenu *sub = i->popup();
			pm->removeItem(mi->id);
			if(sub)
				delete sub;
		}
		d->menuitems.removeRef(mi);
	}

	delete d;
}

void MAction::addTo(QPopupMenu *pm)
{
	MenuItem *mi = new MenuItem;
	connect(pm, SIGNAL(destroyed()), this, SLOT(popup_destroyed()));
	mi->popup = pm;

	insertItem(mi);

	d->menuitems.append(mi);
}

void MAction::insertItem(MenuItem *mi, int index)
{
	QPopupMenu *pm = mi->popup;

	// one or none ?
	if(d->psi->accountList().count() < 2) {
		if(d->iconSet.isNull())
			mi->id = pm->insertItem(d->str, this, SLOT(itemActivated()), 0, -1, index);
		else
			mi->id = pm->insertItem(d->iconSet, d->str, this, SLOT(itemActivated()), 0, -1, index);
		if(d->psi->accountList().isEmpty())
			pm->setItemEnabled(mi->id, false);
	}
	// two or more?
	else {
		QPopupMenu *sub = new QPopupMenu(pm);
		PsiAccountListIt it(d->psi->accountList());
		int n = 0;
		for(PsiAccount *pa; (pa = it.current()); ++it)
			sub->insertItem(pa->name(), n++);

		if(d->iconSet.isNull())
			mi->id = pm->insertItem(d->str, sub, -1, index);
		else
			mi->id = pm->insertItem(d->iconSet, d->str, sub, -1, index);
		connect(sub, SIGNAL(activated(int)), SLOT(menuItemActivated(int)));
	}
}

void MAction::removeFrom(QPopupMenu *pm)
{
	disconnect(pm, SIGNAL(destroyed()), this, SLOT(popup_destroyed()));
	QPtrListIterator<MenuItem> it(d->menuitems);
	for(MenuItem *mi; (mi = it.current());) {
		if(mi->popup == pm) {
			QMenuItem *i = pm->findItem(mi->id);
			if(i) {
				QPopupMenu *sub = i->popup();
				pm->removeItem(mi->id);
				if(sub)
					delete sub;
			}
			d->menuitems.removeRef(mi);
		}
		else
			++it;
	}
}

int MAction::id() const
{
	return d->id;
}

void MAction::itemActivated()
{
	PsiAccount *pa = d->psi->accountList().getFirst();
	if(!pa)
		return;
	doActivate(pa);
}

void MAction::menuItemActivated(int id)
{
	PsiAccount *pa = 0;
	PsiAccountListIt it(d->psi->accountList());
	int n = 0;
	for(; (pa = it.current()); ++it) {
		if(n == id)
			break;
		++n;
	}
	if(!pa)
		return;
	doActivate(pa);
}

void MAction::popup_destroyed()
{
	QPopupMenu *pm = (QPopupMenu *)sender();
	QPtrListIterator<MenuItem> it(d->menuitems);
	for(MenuItem *mi; (mi = it.current());) {
		if(mi->popup == pm)
			d->menuitems.removeRef(mi);
		else
			++it;
	}
}

void MAction::doActivate(PsiAccount *pa)
{
	activated(pa, d->id);
}

void MAction::numAccountsChanged()
{
	// regenerate each menu item
	QPtrListIterator<MenuItem> it(d->menuitems);
	for(MenuItem *mi; (mi = it.current()); ++it) {
		QPopupMenu *pm = mi->popup;
		QMenuItem *i = pm->findItem(mi->id);
		QPopupMenu *sub = i->popup();
		int index = pm->indexOf(mi->id);
		if(index == -1)
			continue;
		pm->removeItem(mi->id);
		if(sub)
			delete sub;
		insertItem(mi, index);
	}
}


//----------------------------------------------------------------------------
// MTray
//----------------------------------------------------------------------------
MTray::MTray(const QImage &icon, const QString &tip, QPopupMenu *popup, QObject *parent)
:QObject(parent)
{
	anim = 0;

	trayTimer = new QTimer(this);
	connect(trayTimer, SIGNAL(timeout()), SLOT(animate()));

	ti = new TrayIcon(makeIcon(icon), tip, popup);
	connect(ti, SIGNAL(clicked(const QPoint &, int)), SIGNAL(clicked(const QPoint &, int)));
	connect(ti, SIGNAL(doubleClicked(const QPoint &)), SIGNAL(doubleClicked(const QPoint &)));
	connect(ti, SIGNAL(closed()), SIGNAL(closed()));
	ti->show();
}

MTray::~MTray()
{
	delete ti;
}

void MTray::setToolTip(const QString &str)
{
	ti->setToolTip(str);
}

void MTray::setImage(const QImage &i)
{
	trayTimer->stop();
	anim = 0;
	ti->setIcon(makeIcon(i));
}

void MTray::setAnim(Anim *a)
{
	anim = a;
	animStep = 0;
	if(option.alertStyle == 0)
		ti->setIcon(makeIcon(anim->base()));

	if(!trayTimer->isActive())
		trayTimer->start(120);
}

bool MTray::isAnimating() const
{
	return (anim ? true: false);
}

void MTray::show()
{
	ti->show();
}

void MTray::hide()
{
	ti->hide();
}

QPixmap MTray::makeIcon(const QImage &_in)
{
#ifdef Q_WS_X11
	/*if(1) {
		QImage real = wmdock[0];
		real.detach();
		QImage in = _in;
		in.detach();

		// make sure it is no bigger than 16x16
		if(in.width() > 16 || in.height() > 16)
			in = in.smoothScale(16,16);

		int xo = 40;
		int yo = 40;

		int n, n2;

		// draw a border
		for(n = 0; n < 22; ++n) {
			real.setPixel(n + xo, yo, qRgb(0x80,0x80,0x80));
			real.setPixel(n + xo, yo + 21, qRgb(0x00,0x00,0x00));
			real.setPixel(xo, yo + n, qRgb(0x80,0x80,0x80));
			real.setPixel(xo + 21, yo + n, qRgb(0x00,0x00,0x00));
		}

		xo += 3;
		yo += 3;

		// draw a dropshadow
		for(n2 = 0; n2 < in.height(); ++n2) {
			for(n = 0; n < in.width(); ++n) {
				if(qAlpha(_in.pixel(n,n2))) {
					real.setPixel(n + xo + 2, n2 + yo + 2, qRgb(0x80,0x80,0x80));
				}
			}
		}
		for(n2 = 0; n2 < in.height(); ++n2) {
			for(n = 0; n < in.width(); ++n) {
				if(qAlpha(_in.pixel(n,n2))) {
					real.setPixel(n + xo, n2 + yo, _in.pixel(n,n2));
				}
			}
		}

		QPixmap icon;
		icon.convertFromImage(real);
		return icon;
	}
	else {*/
		// on X11, the KDE dock is 22x22.  let's make our icon "seem" bigger.
		QImage real(22,22,32);
		//QImage in = _in.convertToImage();
		QImage in = _in;
		in.detach();
		real.fill(0);
		real.setAlphaBuffer(true);

		// make sure it is no bigger than 16x16
		if(in.width() > 16 || in.height() > 16)
			in = in.smoothScale(16,16);

		int xo = (real.width() - in.width()) / 2;
		int yo = (real.height() - in.height()) / 2;

		int n, n2;

		// draw a border
		/*for(n = 0; n < 22; ++n) {
			real.setPixel(n,  0, qRgb(0x80,0x80,0x80));
			real.setPixel(n, 21, qRgb(0x00,0x00,0x00));
			real.setPixel( 0, n, qRgb(0x80,0x80,0x80));
			real.setPixel(21, n, qRgb(0x00,0x00,0x00));
		}*/

		// draw a dropshadow
		for(n2 = 0; n2 < in.height(); ++n2) {
			for(n = 0; n < in.width(); ++n) {
				if(qAlpha(in.pixel(n,n2))) {
					int x = n + xo + 2;
					int y = n2 + yo + 2;
					real.setPixel(x, y, qRgba(0x80,0x80,0x80,0xff));
				}
			}
		}
		// draw the image
		for(n2 = 0; n2 < in.height(); ++n2) {
			for(n = 0; n < in.width(); ++n) {
				if(qAlpha(in.pixel(n,n2))) {
					QRgb c = in.pixel(n, n2);
					real.setPixel(n + xo, n2 + yo, qRgba(qRed(c), qGreen(c), qBlue(c), 0xff));
				}
			}
		}
		// create the alpha layer
		for(n2 = real.height()-2; n2 >= 0; --n2) {
			for(n = 0; n < real.width(); ++n) {
				uint c = real.pixel(n, n2);
				if(c > 0) {
					QRgb old = real.pixel(n, n2);
					real.setPixel(n, n2, qRgba(qRed(old), qGreen(old), qBlue(old), 0xff));
				}
			}
		}

		QPixmap icon;
		icon.convertFromImage(real);
		return icon;
	//}
#else
	QPixmap icon;
	icon.convertFromImage(_in);
	return icon;
#endif
}

void MTray::animate()
{
	if(anim->isAnim() && option.alertStyle == 2) {
		ti->setIcon(makeIcon(anim->frame(animStep)));

		++animStep;
		if(animStep > anim->numFrames()-1)
			animStep = 0;
	}
	else if(option.alertStyle != 0) {
		if(animStep == 0)
			ti->setIcon(makeIcon(anim->base()));
		else if(animStep == 4) {
			QImage blank(16,16,32);
			blank.fill(0);
			blank.setAlphaBuffer(true);
			ti->setIcon(makeIcon(blank));
		}

		++animStep;
		if(animStep == 8)
			animStep = 0;
	}
}
